#PAPER CODE for ALTIERI, COCCHI, ROLI (2023) 
#Efficient computation of spatial entropy measures

#first: set the working directory to your local folder
library(spatstat)
library(SpatEntropy) 

######################
####SECTION 1

data(gorillas)
gorillas

#unmarked pattern
ungorillas = unmark(gorillas)
plot.ppp(ungorillas, pch = 16, main = "Gorilla nesting sites")

#group mark
table(marks(gorillas)$group)
gorillasgroup=gorillas
marks(gorillasgroup) = marks(gorillas)[,1]
plot.ppp(gorillasgroup, pch = 19, cols = 1:2, main="Point data, mark = group")

#covariates
elev = gorillas.extra$elevation
water = gorillas.extra$waterdist
veget = gorillas.extra$vegetation
veget$v[veget$v == "Colonising" | veget$v == "Transition"] = "Primary"
veget$v = as.factor(as.character(veget$v))

plot(elev, col = terrain.colors(100))
plot(veget, col=terrain.colors(length(table(veget$v))))
plot(water, col=topo.colors(1000))

par(mfrow=c(1,3), mar=c(3,3,1,1)) #3 panels
boxplot(c(elev$v), main = "Elevation", ylab = "metres")
boxplot(c(water$v), main = "Water distance", ylab = "metres")
barplot(table(veget$v), main = "Vegetation", ylab = "pixels",
  names.arg = c("Dist", "Grass", "Prim", "Sec"))
par(mfrow=c(1,1))

#make all windows equal
gorillas$window$xrange = elev$xrange
gorillas$window$yrange = elev$yrange
ungorillas$window$xrange = elev$xrange
ungorillas$window$yrange = elev$yrange
gorillasgroup$window$xrange = elev$xrange
gorillasgroup$window$yrange = elev$yrange

area.owin(gorillas$window)        #square metres
area.owin(gorillas$window)/1000^2 #square kilometres

#discrete data
discrgorillas = pixellate(ungorillas, W = gorillas$window,
                          dimyx = elev$dim)
discrgorillas$v[discrgorillas$v >= 1] = 1
table(discrgorillas$v, useNA="ifany")
plot(discrgorillas, col = gray(seq(0.9, 0.1, l=2)), ribbon = F, 
     main = "Grid data, unmarked")

#shannon entropy
shannon(discrgorillas$v) #entropy of nest/no nest
shannon(gorillasgroup)   #entropy of group size


######################
####SECTION 2

#random partition
HB = batty(ungorillas)
HL = karlstrom(ungorillas) # alias of battyLISA(ungorillas)

#vegetation-based partition
HBveg = batty(ungorillas, partition = veget)
HLveg = karlstrom(ungorillas, partition = veget, neigh = 1)

#building the area partition from a continuous covariate
#using the distribution quartiles
HBelev = batty(ungorillas, 
               partition = cut(elev, breaks=quantile(elev$v, na.rm=T)))
HLelev = karlstrom(ungorillas, 
                   partition = cut(elev, breaks=quantile(elev$v, na.rm=T)), 
                   neigh = 1)

HBwat = batty(ungorillas, 
               partition = cut(water, breaks=quantile(water$v, na.rm=T)))
HLwat = karlstrom(ungorillas, 
                   partition = cut(water, breaks=quantile(water$v, na.rm=T)), 
                   neigh = 1)

#plot all partitions together
par(mfrow=c(1,4), mar=c(0,0,0,0))
plot(veget, col=rep("white",4), ribbon=F, main="")
plot(dirichlet(areapart(ungorillas,G=10, plotout=F)$G.pp),
     main="", add=T)
plot(ungorillas, add=T, pch=16)#, cex=.6)
plot(veget, col=terrain.colors(4), ribbon=F, main="")
plot(ungorillas, add=T, pch=16)#, cex=.6)
plot(cut(elev, breaks=quantile(elev$v, na.rm=T)),
     ribbon=F, col=terrain.colors(4), main="")
plot(ungorillas, add=T, pch=16)#, cex=.6)
plot(cut(water, breaks=quantile(water$v, na.rm=T)),
     ribbon=F, col=topo.colors(4), main="")
plot(ungorillas, add=T, pch=16)#, cex=.6)
par(mfrow=c(1,1), mar=c(3,3,3,3))

######################
####SECTION 3

oneill(discrgorillas$v)
contagion(discrgorillas$v)
parredw(discrgorillas$v)

#examples for leibovici's entropy
discrleib=leibovici(discrgorillas$v,
                    cell.size = c(discrgorillas$xstep, discrgorillas$ystep), 
                    ccdist = median(c(pairdist(ungorillas))),
                    verbose = T)

contleib=leibovici(gorillasgroup, ccdist=500) 
#the distance is in metres

#plot of examples for leibovici's distances
par(mfrow=c(1,2), mar=c(0,0,0,0))
plot(veget, col=rep("white",4), ribbon=F, main="")
plot(gorillasgroup, cols=(1:(length(unique(marks(gorillasgroup)))+1))[-2],
     main="", pch=16, cex=.5, legend=F, add=T)
plot(gorillas$window, add=T)
plot(ppp(centroid.owin(gorillas$window)$x,
         centroid.owin(gorillas$window)$y,
         window=gorillas$window), add=T, pch="*", cex=1.5, col=2)
plot(disc(radius = 500, centre = centroid.owin(gorillas$window)),
     add=T, border=2, lwd=2)
plot(discrgorillas, col = gray(seq(0.9, 0.1, l=2)), 
     ribbon = F, main = "")
plot(gorillas$window, add=T)
plot(ppp(centroid.owin(gorillas$window)$x,
         centroid.owin(gorillas$window)$y,
         window=gorillas$window), add=T, pch="*", cex=1.5, col=2)
plot(disc(radius = median(c(pairdist(ungorillas))), centre = centroid.owin(gorillas$window)),
     add=T, border=2, lwd=2)

######################
####SECTION 4

discraltieri=altieri(discrgorillas$v,
        cell.size = c(discrgorillas$xstep, discrgorillas$ystep),
        verbose = T)

contaltieri=altieri(gorillasgroup, distbreak = quantile(c(pairdist(ungorillas)), 
                                             c(.05,.1,.25,.5,.75)))
